function [V y] = BDToptions(K,M,N,Mat,B,Q,d,PutCall,EuroAmer);

% Prices of American and European calls and puts on the zero coupon bonds
% priced from the Black-Derman-Toy binomial tree.
% Required inputs.
% K   = strike price (scalar).
% M   = Maturity of the zero coupon bond (scalar).
% N   = Number of time steps in the BDT tree (scalar).
% Mat = Maturity of the option (scalar).
% B   = Zero coupon bond tree.  Matrix size 2*M+1 by M+1.
% Q   = Arrow-Debreu tree.  Matrix size 2*N+1 by N+1.
% d   = Discount factor tree from BDT.  Matrix size 2*N+1 by N+1.
% PutCall  = 'P'ut or 'C'all.
% EuroAmer = 'E'uropean or 'A'merican.

% Initialize the option values (V) and rescale the zero coupon bond 
% prices (B) and the Arrow Debreu securities (Q) to a size corresponding
% to the maturity (Mat).
V = zeros(2*Mat+1,Mat+1);
B = B(M-Mat+1:M+Mat+1,1:Mat+1);
Q = Q(N-Mat+1:N+Mat+1,1:Mat+1);

% Indices for the nodes at the terminal column.
I = [1:2:2*Mat+1];

% Terminal prices of the option
if strcmp(PutCall,'P')
	V(I,end) = max(K - B(I,end), 0);
elseif strcmp(PutCall,'C')
	V(I,end) = max(B(I,end) - K, 0);
end

if strcmp(EuroAmer,'E')  % European
	y = dot(V(:,end),Q(:,end));
else                     % American
	for j=Mat:-1:1
		for i=Mat-j+2:2:Mat+j
			if strcmp(PutCall,'P')
				V(i,j) = max(K - B(i,j), 0.5*d(i,j)*(V(i-1,j+1) + V(i+1,j+1)));
			elseif strcmp(PutCall,'C')
				V(i,j) = max(B(i,j) - K, 0.5*d(i,j)*(V(i-1,j+1) + V(i+1,j+1)));
			end
		end
	end
	y = V(Mat+1,1);
end
